@@ -78,8 +78,7 @@ defaults.text_spec_tok = [UNK, PAD, BOS, EOS, FLD, TK_MAJ, TK_UP, TK_REP, TK_WRE
 
 
 class BaseTokenizer:
-    """Basic class for a tokenizer function.
-    """
+    """Basic class for a tokenizer function."""
 
     def __init__(self, lang: str):
         self.lang = lang
@@ -278,8 +277,7 @@ class Tokenizer:
         return toks
 
     def _process_all_1(self, texts: Collection[str]) -> List[List[str]]:
-        """Process a list of ``texts`` in one process.
-        """
+        """Process a list of ``texts`` in one process."""
 
         tok = self.tok_func(self.lang)
         if self.special_cases:
@@ -295,8 +293,7 @@ class Tokenizer:
         >>> texts = ['Machine learning is great', 'but building stuff is even better']
         >>> tok = Tokenizer()
         >>> tok.process_all(texts)
-            [['xxmaj', 'machine', 'learning', 'is', 'great'],
-            ['but', 'building', 'stuff', 'is', 'even', 'better']]
+        [['xxmaj', 'machine', 'learning', 'is', 'great'], ['but', 'building', 'stuff', 'is', 'even', 'better']]
 
         .. note:: Note the token ``TK_MAJ`` (`xxmaj`), used to indicate the
             next word begins with a capital in the original text. For more
@@ -332,13 +329,11 @@ class Vocab:
         self.stoi = defaultdict(int, {v: k for k, v in enumerate(self.itos)})
 
     def numericalize(self, t: Collection[str]) -> List[int]:
-        """Convert a list of str (or tokens) ``t`` to their ids.
-        """
+        """Convert a list of str (or tokens) ``t`` to their ids."""
         return [self.stoi[w] for w in t]
 
     def textify(self, nums: Collection[int], sep=" ") -> List[str]:
-        """Convert a list of ``nums`` (or indexes) to their tokens.
-        """
+        """Convert a list of ``nums`` (or indexes) to their tokens."""
         return sep.join([self.itos[i] for i in nums]) if sep is not None else [self.itos[i] for i in nums]  # type: ignore
 
     def __getstate__(self):
@@ -373,9 +368,6 @@ class Vocab:
         >>> texts = ['Machine learning is great', 'but building stuff is even better']
         >>> tokens = Tokenizer().process_all(texts)
         >>> vocab = Vocab.create(tokens, max_vocab=18, min_freq=1)
-        >>> print(vocab.itos)
-        ['xxunk', 'xxpad', 'xxbos', 'xxeos', 'xxfld', 'xxmaj', 'xxup', 'xxrep', 'xxwrep',
-        'is', 'machine', 'learning', 'great', 'but', 'building', 'stuff', 'even', 'better']
         >>> vocab.numericalize(['machine', 'learning', 'is', 'great'])
         [10, 11, 9, 12]
         >>> vocab.textify([10, 11, 9, 12])
